diff --git a/core/modules/field/tests/src/Kernel/FieldImportDeleteTest.php b/core/modules/field/tests/src/Kernel/FieldImportDeleteTest.php
index 0eef137165..f3408a52a0 100644
--- a/core/modules/field/tests/src/Kernel/FieldImportDeleteTest.php
+++ b/core/modules/field/tests/src/Kernel/FieldImportDeleteTest.php
@@ -3,6 +3,7 @@
 namespace Drupal\Tests\field\Kernel;
 
 use Drupal\Component\Utility\SafeMarkup;
+use Drupal\entity_test\Entity\EntityTest;
 use Drupal\field\Entity\FieldConfig;
 use Drupal\field\Entity\FieldStorageConfig;
 
@@ -50,6 +51,14 @@ public function testImportDelete() {
     $field_config_name_2a = "field.field.$field_id_2a";
     $field_config_name_2b = "field.field.$field_id_2b";
 
+    // Create an entity with data in the first field to make sure that field
+    // needs to be purged.
+    $entity_test = EntityTest::create([
+      'type' => 'entity_test',
+    ]);
+    $entity_test->set($field_name, 'test data');
+    $entity_test->save();
+
     // Create a second bundle for the 'Entity test' entity type.
     entity_test_create_bundle('test_bundle');
 
@@ -97,10 +106,10 @@ public function testImportDelete() {
     $this->assertIdentical($active->listAll($field_config_name_2a), []);
     $this->assertIdentical($active->listAll($field_config_name_2b), []);
 
-    // Check that the storage definition is preserved in state.
+    // Check that only the first storage definition is preserved in state.
     $deleted_storages = \Drupal::state()->get('field.storage.deleted') ?: [];
     $this->assertTrue(isset($deleted_storages[$field_storage_uuid]));
-    $this->assertTrue(isset($deleted_storages[$field_storage_uuid_2]));
+    $this->assertFalse(isset($deleted_storages[$field_storage_uuid_2]));
 
     // Purge field data, and check that the storage definition has been
     // completely removed once the data is purged.
diff --git a/core/modules/media/media.install b/core/modules/media/media.install
index 3ff51594b9..86a95188d1 100644
--- a/core/modules/media/media.install
+++ b/core/modules/media/media.install
@@ -36,15 +36,6 @@ function media_install() {
 }
 
 /**
- * Implements hook_uninstall().
- *
- * @TODO Remove when https://www.drupal.org/node/2884202 is fixed.
- */
-function media_uninstall() {
-  \Drupal::moduleHandler()->invoke('field', 'cron');
-}
-
-/**
  * Implements hook_requirements().
  */
 function media_requirements($phase) {
diff --git a/core/modules/system/tests/src/Kernel/Installer/UninstallKernelTest.php b/core/modules/system/tests/src/Kernel/Installer/UninstallKernelTest.php
new file mode 100644
index 0000000000..a4e696962d
--- /dev/null
+++ b/core/modules/system/tests/src/Kernel/Installer/UninstallKernelTest.php
@@ -0,0 +1,44 @@
+<?php
+
+namespace Drupal\Tests\system\Kernel\Module;
+
+use Drupal\KernelTests\KernelTestBase;
+
+/**
+ * Tests the uninstallation of modules.
+ *
+ * @group Module
+ */
+class UninstallKernelTest extends KernelTestBase {
+
+  /**
+   * Modules to enable.
+   *
+   * @var array
+   */
+  public static $modules = ['system', 'user', 'field', 'file', 'image', 'media'];
+
+  /**
+   * {@inheritdoc}
+   */
+  protected function setUp() {
+    parent::setUp();
+
+    $this->installSchema('user', ['users_data']);
+    $this->installEntitySchema('media');
+    $this->installEntitySchema('file');
+    $this->installConfig(['media']);
+  }
+
+  /**
+   * Tests uninstalling media and file modules.
+   */
+  public function testUninstallMedia() {
+    // Media creates a file field that is removed on uninstall, ensure that it
+    // is fully deleted (as it is empty) and that file then can be uninstalled
+    // as well.
+    \Drupal::service('module_installer')->uninstall(['media']);
+    \Drupal::service('module_installer')->uninstall(['file']);
+  }
+
+}

diff --git a/core/modules/field/tests/src/Kernel/FieldCrudTest.php b/core/modules/field/tests/src/Kernel/FieldCrudTest.php
index 08fe18c5c2..6407be7520 100644
--- a/core/modules/field/tests/src/Kernel/FieldCrudTest.php
+++ b/core/modules/field/tests/src/Kernel/FieldCrudTest.php
@@ -269,12 +269,11 @@ public function testUpdateField() {
   }
 
   /**
    * Test the deletion of a field.
    */
-  public function testDeleteField() {
-    // TODO: Test deletion of the data stored in the field also.
-    // Need to check that data for a 'deleted' field / storage doesn't get loaded
-    // Need to check data marked deleted is cleaned on cron (not implemented yet...)
+  public function testDeleteFieldNoData() {
+    // Deleting and purging fields with data is tested in
+    // \Drupal\Tests\field\Kernel\BulkDeleteTest.
 
     // Create two fields for the same field storage so we can test that only one
     // is deleted.
@@ -289,17 +288,18 @@ public function testDeleteField() {
     $this->assertTrue(!empty($field) && empty($field->deleted), 'A new field is not marked for deletion.');
     $field->delete();
 
-    // Make sure the field is marked as deleted when it is specifically loaded.
-    $field = current(entity_load_multiple_by_properties('field_config', ['entity_type' => 'entity_test', 'field_name' => $this->fieldDefinition['field_name'], 'bundle' => $this->fieldDefinition['bundle'], 'include_deleted' => TRUE]));
-    $this->assertTrue($field->isDeleted(), 'A deleted field is marked for deletion.');
+    // Make sure the field was deleted without being marked for purging as there
+    // was no data.
+    $fields = entity_load_multiple_by_properties('field_config', ['entity_type' => 'entity_test', 'field_name' => $this->fieldDefinition['field_name'], 'bundle' => $this->fieldDefinition['bundle'], 'include_deleted' => TRUE]);
+    $this->assertEquals(0, count($fields), 'A deleted field is marked for deletion.');
 
     // Try to load the field normally and make sure it does not show up.
     $field = FieldConfig::load('entity_test.' . '.' . $this->fieldDefinition['bundle'] . '.' . $this->fieldDefinition['field_name']);
-    $this->assertTrue(empty($field), 'A deleted field is not loaded by default.');
+    $this->assertTrue(empty($field), 'Field was deleted');
 
     // Make sure the other field is not deleted.
     $another_field = FieldConfig::load('entity_test.' . $another_field_definition['bundle'] . '.' . $another_field_definition['field_name']);
-    $this->assertTrue(!empty($another_field) && empty($another_field->deleted), 'A non-deleted field is not marked for deletion.');
+    $this->assertTrue(!empty($another_field) && !$another_field->isDeleted(), 'A non-deleted field is not marked for deletion.');
   }
 
   /**
@@ -269,12 +269,11 @@ public function testUpdateField() {
 
   /**
-   * Test the deletion of a field.
+   * Test the deletion of a field with no data.
    */